function [a b A Q1 Q2 R1 R2] = CalculateAandB(x,y,z,lambda,detail)

% Calculates required vectors and matrices for thin plate splines
% x = vector of x values (strikes)
% y = vector of y values (maturities)
% z = vector of responses (volatilities)
% lambda = smoothing parameter (0=no smoothing)
% detail = detailed output (Y/N)

% By Fabrice Douglas Rouah, FRouah.com and Volopta.com

% Create the B matrix
N = length(z);
B = [];
for k=1:N
    B = [B; 1 x(k) y(k)];
end

% Create the A matrix
for i=1:N
    for j=1:N
        z1 = x(i)-x(j);
        z2 = y(i)-y(j);
        r = z1^2 + z2^2;
        if r == 0
            A(i,j) = 0;
        else
            A(i,j) = (r)*log(r);
        end
    end
end

% QR decomposition of B and define submatrices
[Q R] = qr(B);
Q1 = Q(:,1:3);
Q2 = Q(:,4:N);
R1 = R(1:3,:);
R2 = R(4:N,:);
Q2T = transpose(Q2);

% Identity matrix
I = eye(N);

% Calculate the a and b vectors
a = Q2 / (Q2T*(A+(N*lambda).*I)*Q2) * (Q2T*z);
b = inv(R1)*transpose(Q1)*(z - A*a);

% Detailed output on checks and validations
if strcmp(detail,'Y')
    fprintf('Condition number on A is %10.2e \n',cond(A))
    fprintf('Checks on the QR decomposition, each should be zero \n');
    fprintf('--------------------------------------------------- \n')
    fprintf('Sum(a)   %10.2e \n', sum(a))
    fprintf('x*a      %10.2e \n', dot(x,a))
    fprintf('y*a      %10.2e \n', dot(y,a))
    fprintf('--------------------------------------------------- \n')
end
