% Thin plate splines, example from Cont and de Fontoura
% By Fabrice Douglas Rouah, FRouah.com and Volopta.com

clc; clear;

% Input the axes x,y and the points f=z
x = [-1  0  0  1  0];
y = [ 0 -1  1  0  0];
z = [ 4  5  3  4  2]';

% Arrange into a matrix and vector a plot later
fmat  = [nan 4 nan; 5 2 3; nan 4 nan];
xvec = [-1 0 1];
yvec = [-1 0 1];

% Smoothing parameter
lambda = 0;

% Build the mesh
dx = .1;
dy = .1;
xi = [2.5*min(x):dx:2.5*max(x)]';
yi = [2.5*min(y):dy:2.5*max(y)]';
Nxi = length(xi);
Nyi = length(yi);

% Obtain the vectors "a" and "b" and request detailed output
detail = 'Y';
[a b A Q1 Q2 R1 R2] = CalculateAandB(x,y,z,lambda,detail);

% Calculate the thin plate interpolated points
for i=1:Nxi
    for j=1:Nyi;
        zi(i,j) = TPSpoint(xi(i),yi(j),x,y,a,b);
    end
end

% Plot everything
surf(yi,xi',zi)
hold on
mesh(xvec,yvec,fmat,'Marker','o','MarkerFaceColor','black','MarkerSize',6)
alpha(0.5)
hold off
view([1 -2 1])

