% Thin plate spline smoothing of the SP500 implied vol surface
% By Fabrice Douglas Rouah, FRouah.com and Volopta.com

clc; clear;

% Implied volatility: Rows are maturities, so transpose
IV = [ ...
   27.8000   26.3800   25.3200   25.1800
   24.7700   24.0200   23.6400   23.6900
   21.8600   21.5800   22.0300   22.3900
   18.7800   19.3000   20.4700   20.9800
   15.7200   17.1200   18.9400   19.7000
   13.3400   15.1700   17.4800   18.4900
   13.2300   13.7300   16.1800   17.3600]./100;
T = [45 98 261 348]./365;
K = [120   125   130   135   140   145   150];
[NK NT] = size(IV);

% Smoothing parameter
lambda = 0.1;

% Build the mesh on which the TPS is calculated
Nxi = 50;
Nyi = 25;
dx = (K(end)-K(1))/(Nxi-1);
dy = (T(end)-T(1))/(Nyi-1);
Ki = K(1):dx:K(end);
Ti = T(1):dy:T(end);

% Obtain the TPS
detail = 'Y';
IVi = TPS(K,T,IV,Ki,Ti,lambda,detail);

% Arrange the market strikes, vols, and maturities into a vector
z = IV(:);
[Y X] = meshgrid(T,K);
x = X(:)';
y = Y(:)';

% Plot everything
surf(Ki,Ti',IVi')
hold on
plot3(x,y,z,'wo','Marker','o','MarkerFaceColor','black','MarkerSize',6)
alpha(0.5)
hold off
view([1 -1 1])

